<?php

namespace HCM\Geral\Controllers;

use Doctrine\ORM\EntityManager;
use HCM\Geral\Services\EsocialBxService;
use HCM\Geral\Logic\ESocialLogic;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource\DataSet;

class EsocialBxController extends Crud {

    protected $dataSourceName = 'setEsoBxEsocial';
    protected $dataSourceNameGet = 'getEsoBxEsocial';
    protected $dataSourceNameGetPerApur = 'getPeriodoApuracaoEsocial';
    protected $dataSourceNameGetRegistro = 'getEsoEventoBxEsocial';
    protected $dataSourceNameSetRegistro = 'setEsoEventoBxEsocial';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, EntityManager $entityManager, HCMParameters $hcmParameters, EsocialBxService $esocialBxService){
        $this->parameterBag = $parameterBag;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->esocialBxService = $esocialBxService;
        parent::__construct($dataSourceManager);
    }

    public function getEsoBxEsocial(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceNameGet);
            $filterData->addCondition('P_NRORG','=',$this->hcmParameters->getNrorgTrabalho());
            $filterData->addCondition('P_NRORG_PADRAO','=',$this->hcmParameters->getNrorgPadrao());
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $rows = $dataSet->getRows();
            $response->addDataSet(new DataSet($this->dataSourceNameGet, $rows));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
    public function setEsoBxEsocial(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (!isset($row['NREVENTO'])) {
                    $row['NREVENTO'] = null;
                }
                if (!isset($row['DSCHAVE'])) {
                    $row['DSCHAVE'] = null;
                }
                if (!isset($row['DTENVIO'])) {
                    $row['DTENVIO'] = null;
                }
                if (!isset($row['IDATIVO'])) {
                    $row['IDATIVO'] = 'S';
                }
                if (!isset($row['DTRESPOSTA'])) {
                    $row['DTRESPOSTA'] = null;
                }
                if (!isset($row['DTRESPOSTA'])) {
                    $row['DTRESPOSTA'] = null;
                }
                if (!isset($row['CDERRO'])) {
                    $row['CDERRO'] = null;
                }
                if (!isset($row['DSOBSERVACAO'])) {
                    $row['DSOBSERVACAO'] = null;
                }
                if (!isset($row['DSDIRXMLENV'])) {
                    $row['DSDIRXMLENV'] = null;
                }
                if (!isset($row['DSDIRXMLRETENV'])) {
                    $row['DSDIRXMLRETENV'] = null;
                }
                if (!isset($row['DSDIRXMLRETPRO'])) {
                    $row['DSDIRXMLRETPRO'] = null;
                }
                if (!isset($row['DTPROCESSAMENTO'])) {
                    $row['DTPROCESSAMENTO'] = null;
                }
                
                if(array_key_exists('NRCPFTRABALHADOR', $row)){
                    $row['NRCPFTRABALHADOR'] = str_replace('.', '', $row['NRCPFTRABALHADOR']);
                    $row['NRCPFTRABALHADOR'] = str_replace('-', '', $row['NRCPFTRABALHADOR']);
                }
                
                if (array_key_exists('NRBXESOCIAL', $row)) {
                    if ($row['NRBXESOCIAL'] === '') {
                        $row['NRBXESOCIAL'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_BXESOCIAL', 1, 12, true); 
                    }
                } else {
                    $row['NRBXESOCIAL'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_BXESOCIAL', 1, 12, true); 
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteEsoBxEsocial(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    public function getPeriodoApuracaoEsocial(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceNameGetPerApur);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $rows = $dataSet->getRows();
            $response->addDataSet(new DataSet($this->dataSourceNameGetPerApur, $rows));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function cadastroAutomaticoBxEsocial(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row  = $request->getRow();
            
            $nrestruturam = $row['NRESTRUTURAM'];
            $nrtpambiente = $row['NRTPAMBIENTE'];
            $nreventolist = $row['NREVENTOLIST'];
            $cdperiodoapuracaolist = isset($row['CDPERIODOAPURACAOLIST']) ? $row['CDPERIODOAPURACAOLIST'] : array();
            
            $return = $this->esocialBxService->cadastroAutomaticoBxEsocial($nrestruturam, $nrtpambiente, $nreventolist, $cdperiodoapuracaolist);
            
            $response->addDataSet(new DataSet('cadastroAutomaticoBxEsocial', $return));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
    public function gerarTemplatePlanilhaCPF(DTO\Request\Row $request, DTO\Response $response){
        try{
            $row  = $request->getRow();
            
            $file = $this->esocialBxService->gerarTemplatePlanilhaCPF($row);

            if($file !== null){
                $retorno = array('NMARQUIVO' => $file);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $retorno));
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function importarPlanilhaCPFBxEsocial(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row  = $request->getRow();
            
            $nrestruturam = $row['NRESTRUTURAM'];
            $nrtpambiente = $row['NRTPAMBIENTE'];
            $arquivo = $row['DSPATH'][0];
            
            $retorno = $this->esocialBxService->importarPlanilhaCPFBxEsocial($nrestruturam, $nrtpambiente, $arquivo);
            $mensagens = !empty($retorno['messages']) ? $this->esocialBxService->preparaMensagens($retorno['messages']) : null;
            
            if($retorno['success'] == true){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida com sucesso.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            } else {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida. Existem erros na importação", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
            }
            
            if(!empty($mensagens)){
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msg', $mensagens));
            }
            
            $response->addDataSet(new DataSet('importacaoPlanilhaCPFBxEsocial', array(true)));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function consultarIdentificadoresEventos(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row  = $request->getRow();
            
            $retorno = false;
            
            $nrestruturam = $row['NRESTRUTURAM'];
            $nrtpambiente = $row['NRTPAMBIENTE'];
            $nrbxesocial = $row['NRBXESOCIAL'];
            $idtipoconsulta = $row['IDTIPO'];
            
            $logic = new ESocialLogic($this->entityManager, $this->hcmParameters, $nrestruturam, $nrtpambiente);
            
            switch ($idtipoconsulta) {
                case 'E':
                    $retorno = $logic->consultarIdentificadoresEventosEmpregador($nrbxesocial);
                    break;
                case 'T':
                    $retorno = $logic->consultarIdentificadoresEventosTabela($nrbxesocial);
                    break;
                case 'F':
                    $retorno = $logic->consultarIdentificadoresEventosTrabalhador($nrbxesocial);
                    break;
            }
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('consultarIdentificadoresEventos', array('result' => $retorno)));
            
            if ($retorno) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Consulta realizada com sucesso", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            } else {
                if (count($logic->getMessages()) == 1) {
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(current($logic->getMessages()), \Zeedhi\Framework\DTO\Response\Message::TYPE_ERROR));
                } else {
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Consulta com erros", \Zeedhi\Framework\DTO\Response\Message::TYPE_ERROR));
                }
                
                if(!empty($logic->getMessages())){
                    foreach($logic->getMessages() as $message) {
                        $msg[] = array('msg' => $message);
                    }
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msg', $msg));
                }
            }
        } catch(\Exception $e) {
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('consultarIdentificadoresEventos', array('result' => false)));
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getEsoBxEsocialRetorno(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $nrestruturam = null;
            $nrtpambiente = null;
            $nrbxesocial = null;
            $xmlFile = null;
            
            foreach ($request->getFilterCriteria()->getConditions() as $condition) {
                if ($condition['columnName'] == 'NRESTRUTURAM') {
                    $nrestruturam = $this->processConditionValue($condition);
                }
                if ($condition['columnName'] == 'NRTPAMBIENTE') {
                    $nrtpambiente = $this->processConditionValue($condition);
                }
                if ($condition['columnName'] == 'NRBXESOCIAL') {
                    $nrbxesocial = $this->processConditionValue($condition);
                }
            }
            
            $logic = new ESocialLogic($this->entityManager, $this->hcmParameters, $nrestruturam, $nrtpambiente);
            
            $bxEsocial = $logic->getBxesocial($nrbxesocial);
            if(is_object($bxEsocial)){
                $xmlFile = $bxEsocial->getDsdirxmlretenv();
            }
            
            $xml = $this->esocialBxService->getXmlFiles($nrorg, $xmlFile, null, null, null);
            
            $xmlDataSet = $logic->setXmlFileToDataSet($xml);
            $dataSet = new DataSet($request->getRoutePath(), $xmlDataSet);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($xmlFile, $e->getCode()));
        }
    }

    public function getEsoEventoBxEsocial(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceNameGetRegistro);
            $filterData->addCondition('P_NRORG','=',$this->hcmParameters->getNrorgTrabalho());
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $rows = $dataSet->getRows();
            $response->addDataSet(new DataSet($this->dataSourceNameGet, $rows));
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
    public function setEsoEventoBxEsocial(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = $this->dataSourceNameSetRegistro;
            foreach ($request->getDataSet()->getRows() as &$row) {
                if ((!isset($row['IDEVENTO']) || $row['IDEVENTO'] == "") && (!isset($row['NRRECIBO']) || $row['NRRECIBO'] == "")) {
                    $response->addNotification(new DTO\Response\Notification('Deve ser informado pelo menos o ID Evento ou o Recibo de Processamento', DTO\Response\Notification::TYPE_ERROR));
                    return;
                }
                if (!isset($row['NREVENTO'])) {
                    $row['NREVENTO'] = null;
                }
                if (!isset($row['DSDIRXMLEVENTO'])) {
                    $row['DSDIRXMLEVENTO'] = null;
                }
                if (!isset($row['IDATIVO'])) {
                    $row['IDATIVO'] = 'S';
                }
                if (array_key_exists('NREVENTOBXESOCIAL', $row)) {
                    if ($row['NREVENTOBXESOCIAL'] === '') {
                        $row['NREVENTOBXESOCIAL'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_EVENTOBXESOCIAL', 1, 12, true); 
                    }
                } else {
                    $row['NREVENTOBXESOCIAL'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_EVENTOBXESOCIAL', 1, 12, true); 
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteEsoEventoBxEsocial(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    public function downloadEventosPorId(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row  = $request->getRow();
            
            $retorno = false;
            
            $nrestruturam = $row['NRESTRUTURAM'];
            $nrtpambiente = $row['NRTPAMBIENTE'];
            $nrbxesocial = $row['NRBXESOCIAL'];
            $eventos = $row['ESOEVENTOBXESOCIAL'];
            
            $logic = new ESocialLogic($this->entityManager, $this->hcmParameters, $nrestruturam, $nrtpambiente);
            
            $retorno = $logic->downloadEventosPorId($nrbxesocial, $eventos);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('downloadEventosPorId', array('result' => $retorno)));
            
            if ($retorno) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Download realizado com sucesso", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            } else {
                if (count($logic->getMessages()) == 1) {
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(current($logic->getMessages()), \Zeedhi\Framework\DTO\Response\Message::TYPE_ERROR));
                } else {
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Download com erros", \Zeedhi\Framework\DTO\Response\Message::TYPE_ERROR));
                }
                
                if(!empty($logic->getMessages())){
                    foreach($logic->getMessages() as $message) {
                        $msg[] = array('msg' => $message);
                    }
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msg', $msg));
                }
            }
        } catch(\Exception $e) {
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('downloadEventosPorId', array('result' => false)));
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function downloadEventosPorNrRecibo(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row  = $request->getRow();
            
            $retorno = false;
            
            $nrestruturam = $row['NRESTRUTURAM'];
            $nrtpambiente = $row['NRTPAMBIENTE'];
            $nrbxesocial = $row['NRBXESOCIAL'];
            $eventos = $row['ESOEVENTOBXESOCIAL'];
            
            $logic = new ESocialLogic($this->entityManager, $this->hcmParameters, $nrestruturam, $nrtpambiente);
            
            $retorno = $logic->downloadEventosPorNrRecibo($nrbxesocial, $eventos);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('downloadEventosPorNrRecibo', array('result' => $retorno)));
            
            if ($retorno) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Download realizado com sucesso", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            } else {
                if (count($logic->getMessages()) == 1) {
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(current($logic->getMessages()), \Zeedhi\Framework\DTO\Response\Message::TYPE_ERROR));
                } else {
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Download com erros", \Zeedhi\Framework\DTO\Response\Message::TYPE_ERROR));
                }
                
                if(!empty($logic->getMessages())){
                    foreach($logic->getMessages() as $message) {
                        $msg[] = array('msg' => $message);
                    }
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msg', $msg));
                }
            }
        } catch(\Exception $e) {
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('downloadEventosPorNrRecibo', array('result' => false)));
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 